#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2009  Olaf Merkert


# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see
# <http://www.gnu.org/licenses/>.


# README:

# This little python script downloads the latest movie trailer from
# www.moviemaze.de into ~/Trailers.  Make sure to create this
# directory before you start the script.  The ~ corresponds to what
# your HOME environment variable contains.  If you really don't know
# where that is, run the script from a commandline interpreter and
# consult the output or change the source to use a different
# directory.

# Dependencies are only python (tested with 2.5 on linux) and numpy.

# If you have any questions, suggestions or patches, feel free to send
# me an email to olaf@m-merkert.de.

import os
import os.path as path
import re
import sys
from urllib import urlretrieve, urlopen
from numpy import array
from xml.etree.ElementTree import parse

def select_size(available, preferred = 850):
    z = array(available, dtype="int") - preferred
    weighted = (z**4 - z**3 + z**2)
    i = weighted.argmin()
    return available[i]

def select_lang(available, preferred = ["en", "de"]):
    if len(available) == 1:
        return available[0]
    for p in preferred:
        if p in available:
            return p
    return available[0]

def get_trailer_list(feed_url):
    # trailers = []
    # f = None
    try:
        f = urlopen(feed_url)
        rsstree = parse(f)
        items = rsstree.findall("//item")
        trailers = [ [item.findtext("title").partition(" zu ")[2],
                      item.findtext("link")]
                     for item in items]
    finally:
        if f: f.close()
    return trailers

def dump_url(url):
    dump = ""
    try:
        stream = urlopen(url)
        dump = stream.read()
    finally:
        if stream: stream.close()
    return dump

gtv_len = len("generateDownloadLink")
gtv_re = re.compile("generateDownloadLink\(\"[^\"]*\"\)")

def get_trailer_versions(page_url):
    contents = dump_url(page_url)
    versions = {}
    for link in re.findall(gtv_re, contents):
        link_stripped = link[gtv_len+2:-2]
        core_name = link_stripped.rpartition("/")[2].rpartition(".")[0]
        score_parts = core_name.split("_")
        size = score_parts[-1]
        version_qual = " ".join(score_parts[1:-1])
        version, underscore, lang = version_qual.rpartition("-")
        if version not in versions:
            versions[version] = {}
        if lang not in versions[version]:
            versions[version][lang] = {}
        versions[version][lang][size] = link_stripped
    return versions

def download_from_feed(feed_url):
    titles_and_urls = get_trailer_list(feed_url)
    dnld_collection = []
    for title, page_url in titles_and_urls:
        dnld_collection.extend(
            download_from_page(title, page_url,
                               select_size, select_lang)
            )
    return dnld_collection

dnld_dir = path.expanduser(
    "~/Trailers/")

def calculate_destination(title, dnld_url, version, lang, size):
    ext = dnld_url.rpartition(".")[2]
    fname = title + "-" + "".join([version, lang, size]) + "." + ext
    dest = path.join(dnld_dir, fname)
    return (dest, fname)

def download_from_page(title, page_url, size_selector, lang_selector):
    version_hier = get_trailer_versions(page_url)
    top_level = get_top_level(page_url)
    coll = []
    for n, v in version_hier.iteritems():
        lang = lang_selector(v.keys())
        size = size_selector(v[lang].keys())
        (destination, fname) = calculate_destination(title, v[lang][size], n, lang, size)
        was_new = download_file(top_level, v[lang][size], destination)
        if was_new:
            print "Downloaded %s for %s." % ("Trailer", title)
            coll.append(fname)
        else:
            print "%s for %s is already downloaded." % ("Trailer", title)
    return coll

def get_top_level(url):
    return "/".join(url.split("/")[:3])

def download_file(server_prefix, sub_url, destination):
    if path.exists(destination):
        return False

    if sub_url[0] == "/":
        dnld_url = server_prefix + sub_url
    else:
        dnld_url = sub_url

    def report(count, block_size, total):
        if total != -1 and count % 100 == 0:
            print "%i%%" % (100 * count * block_size // total) ,
            sys.stdout.flush()
    print "Downloading to %s" % destination
    tmp_destination = destination + ".part"
    urlretrieve(dnld_url, tmp_destination, report)
    os.rename(tmp_destination, destination)
    print
    return True


if __name__=="__main__":
    feed_url = "http://www.moviemaze.de/rss/trailer.phtml"
    dnld_c = download_from_feed(feed_url)
    print "New downloads:"
    for d in dnld_c:
        print d
